/* based on: $XConsortium: xload.c,v 1.37 94/04/17 20:43:44 converse Exp $ */
/*

Copyright (c) 1989  X Consortium

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall
not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization
from the X Consortium.

*/

/*

  xpppoem  monitor a DSL connection over pppoe

  by Thomas Schmitt (scdbackup@gmx.net) 

  Published under BSD license which essentially says the same
  than the license terms above.

  Derived from Mario Klebsch's (mario@klebsch.de) xpppload 
  (which he derived from xisdnload which is derived from xload)
  Found as http://www.klebsch.de/xpppload-0.4a.tar.gz  23 Oct 2001

  All PPP specific code was changed to use pppoem which reads the
  files /proc/net/pppoe and /proc/net/dev and also prints statistics
  to stdout and stderr.
  Some features were added like colors indicating throughput and
  adjustable unit for the histogram. Some others were removed like
  the "trying" color or the "activate"/"deactivate" triggers.


  Give xpppload a try if xpppoem does not find the /proc files.
  It uses a completely different approach via sockets and ioctls.
  If i had known xpppload before, pppoem would not have been made.
  They are different enough to coexist, though.

*/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <signal.h>


#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/StripChart.h>
#include <X11/Xmu/SysUtil.h>


#include "xpppload.bit"

char *ProgramName;

static void quit();

/*
 * Definition of the Application resources structure.
 */

typedef struct _XPppoemResources {
  Boolean show_label;
  Boolean show_graph;
  char *online_color;
  char *cps_range;
  char *show;
  char *busy_color;
  char *busy_samples;
  char *offline_color;
  char *unit;
  char *dev_name;
  char *pretend;
  char *listen_stdin;
  char *report;
} XPppoemResources;

/*
 * Command line options table.  Only resources are entered here...there is a
 * pass over the remaining options after XtParseCommand is let loose.
 */

static XrmOptionDescRec options[] = {
 {"-scale",		"*load.minScale",	XrmoptionSepArg,	NULL},
 {"-update",		"*load.update",		XrmoptionSepArg,	NULL},
 {"-hl",		"*load.highlight",	XrmoptionSepArg,	NULL},
 {"-highlight",		"*load.highlight",	XrmoptionSepArg,	NULL},
 {"-nolabel",           "*showLabel",           XrmoptionNoArg,      "False"},
 {"-nograph",           "*showGraph",           XrmoptionNoArg,      "False"},
 {"-jumpscroll",	"*load.jumpScroll",	XrmoptionSepArg,	NULL},
 {"-online",		"*onlineColor",         XrmoptionSepArg,	NULL},
 {"-cps_range",		"*cpsRange",            XrmoptionSepArg,	NULL},
 {"-show",	        "*showSpans",           XrmoptionSepArg,	NULL},
 {"-busy",		"*busyColor",           XrmoptionSepArg,	NULL},
 {"-busy_samples",	"*busySamples",         XrmoptionSepArg,	NULL},
 {"-offline",		"*offlineColor",        XrmoptionSepArg,	NULL},
 {"-unit",		"*unitScale",           XrmoptionSepArg,	NULL},
 {"-dev_name",		"*devName",             XrmoptionSepArg,	NULL},
 {"-pretend",		"*pretendOpen",         XrmoptionSepArg,	NULL},
 {"-listen_stdin",	"*listenStdin",         XrmoptionSepArg,	NULL},
 {"-report",		"*reportText",          XrmoptionSepArg,	NULL}
};

/*
 * The structure containing the resource information for the
 * Xpppload application resources.
 */

#define Offset(field) (XtOffsetOf(XPppoemResources, field))

static XtResource my_resources[] = {
  {"showLabel", XtCBoolean, XtRBoolean, sizeof(Boolean),
     Offset(show_label), XtRImmediate, (XtPointer) TRUE},
  {"showGraph", XtCBoolean, XtRBoolean, sizeof(Boolean),
     Offset(show_graph), XtRImmediate, (XtPointer) TRUE},
  {"onlineColor", "OnlineColor", XtRString, sizeof(char *),
     Offset(online_color), XtRString, NULL},
  {"cpsRange", "CpsRange", XtRString, sizeof(char *),
     Offset(cps_range), XtRString, NULL},
  {"showSpans", "ShowSpans", XtRString, sizeof(char *),
     Offset(show), XtRString, NULL},
  {"busyColor", "BusyColor", XtRString, sizeof(char *),
     Offset(busy_color), XtRString, NULL},
  {"busySamples", "BusySamples", XtRString, sizeof(char *),
     Offset(busy_samples), XtRString, NULL},
  {"offlineColor", "OfflineColor", XtRString, sizeof(char *),
     Offset(offline_color), XtRString, NULL},
  {"unitScale", "unitScale", XtRString, sizeof(char *),
     Offset(unit), XtRString, NULL},
  {"devName", "DevName", XtRString, sizeof(char *),
     Offset(dev_name), XtRString, NULL},
  {"pretendOpen", "PretendOpen", XtRString, sizeof(char *),
     Offset(pretend), XtRString, NULL},
  {"listenStdin", "ListenStdin", XtRString, sizeof(char *),
     Offset(listen_stdin), XtRString, NULL},
  {"reportText", "ReportText", XtRString, sizeof(char *),
     Offset(report), XtRString, NULL},
};

#undef Offset

static XPppoemResources resources;

static XtActionsRec xpppoem_actions[] = {
    { "quit",	quit },
};
static Atom wm_delete_window;


static Pixel onlinecolor, offlinecolor, bgcolor;

static Widget label_wid;


#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <wait.h>
#include <unistd.h>


#define Pppoem_use_own_cleanuP
#define Pppoem_use_own_sfilE
#define Pppoem_no_system_includeS

#include "../pppoem.h"

static struct PppoeM *PppoemM= NULL;

#include "../pppoem.c"

static int Xpppoem_do_show_heaT= 0;
static int Xpppoem_busy_sampleS= 15;
static int Xpppoem_heat_rangE= 90000.0;
static double Xpppoem_uniT= 20000.0;
static int Xpppoem_show_labeL= 1;
static int Xpppoem_show_grapH= 1;


/*
 * Exit with message describing command line format.
 */

void usage(flag)
int flag;
/*
 bit0= short form
*/
{
    fprintf (stderr, "usage:  %s [-options ...]\n", ProgramName);
    fprintf (stderr, "\n");
    fprintf (stderr,
      "xload lookalike program displaying a histogram of DSL throughput.\n");
    if(!(flag&1)) 
      fprintf (stderr, "\n");
    fprintf (stderr, "where options include:\n");
    fprintf (stderr,
      "    -display dpy        X server on which to display\n");
    fprintf (stderr,
      "    -geometry geom      size and location of window\n");
    fprintf (stderr,
      "    -nolabel            do not show a headline over histogram\n");
    fprintf (stderr,
      "    -nograph            do not show histogram but only a color pad\n");
    fprintf (stderr,
      "    -fn font            font to use in label\n");
    fprintf (stderr,
      "    -unit number        data rate representing one scale unit\n");
    if(!(flag&1)) 
      fprintf (stderr,
      "                        the default value is 20000\n");
    fprintf (stderr,
      "    -scale number       minimum number of scale lines\n");
    fprintf (stderr,
      "    -update seconds     interval between updates\n");
    fprintf (stderr,
      "    -bg color           background color of text label\n");
    fprintf (stderr,
      "    -fg color           graph color\n");
    fprintf (stderr,
      "    -hl color           scale and text color\n");
    fprintf (stderr,
      "    -online color       background color when online\n");
    fprintf (stderr,
      "    -busy color|off     indicate throughput by changing colors\n");

    if(!(flag&1)) {
      fprintf (stderr,
      "                        based on -busy_samples average. 0 cps will\n");
      fprintf (stderr,
      "                        show the -online color. The value given\n");
      fprintf (stderr,
      "                        with -cps_range shows the -busy color.\n");
      fprintf (stderr,
      "                        Color \"off\" disables color changing.\n");
    }
    fprintf (stderr,
      "    -busy_samples value  number of samples for changing colors\n");
    if(!(flag&1)) {
      fprintf (stderr,
      "                        (default: 15)\n");
      fprintf (stderr,
      "    -cps_range value    upper cps limit of a color scale defined\n");
      fprintf (stderr,
      "                        with -online and -busy (default: 90000)\n");
      fprintf (stderr,
      "                        Note: if this value is larger than the\n");
      fprintf (stderr,
    "                        largest \"-show\" number, the eternal average\n");
      fprintf (stderr,
      "                        of the connection determines the color.\n");
    } else
      fprintf (stderr,
      "    -cps_range value    upper cps limit of a -busy color scale\n");
    fprintf (stderr,
      "    -offline color      background color when offline\n");
    fprintf (stderr,
      "    -jumpscroll value   number of pixels to scroll on overflow\n");
    if((flag&1)) 
      fprintf (stderr,
      "    -help               print a longer help text on stderr\n");

    if(!(flag&1)) {
      fprintf (stderr, "\n");
      Pppoem__help_stderr(1);
    }

    fprintf (stderr, "\n");
    exit(1);
}


void Xpppoem_init()
{
 int ret;

 ret= Pppoem_new(&PppoemM,"-",1.0,1,15,300,1|2);
 if(ret<=0)
   exit(1);
}


int Xpppoem_start(mon,flag)
struct PppoeM *mon;
int flag;
{
 /* to get a message on exit */
 Cleanup_set_handlers(0);

 Pppoem_report(mon,"PPPOEM S",1|2);
 if(! Pppoem_has_headlines_on(mon,0))
   Pppoem_show_stat(mon,2);                                  /* one headline */
 return(1);
}


unsigned Xpppoem_pixel_to_rgb(w,pixel,flag)
Widget w;
Pixel pixel;
int flag;
{
 XColor color;
 unsigned rgb,red,green,blue;

 color.pixel= onlinecolor;
 XQueryColor(XtDisplay(w),
             DefaultColormap(XtDisplay(w),
                             DefaultScreen(XtDisplay(w))),
             &color);
 red= (color.red>>8)&255;
 green= (color.green>>8)&255;
 blue= (color.blue>>8)&255;
 rgb= (red<<16) | (green<<8) | blue;
 return(rgb);
}


unsigned Xpppoem_colorname_to_rgb(w,colorname,flag)
Widget w;
char *colorname;
int flag;
{
 XColor color;
 unsigned rgb,red,green,blue;

 XParseColor(XtDisplay(w),
             DefaultColormap(XtDisplay(w),
                             DefaultScreen(XtDisplay(w))),
             colorname,
             &color);
 red= (color.red>>8)&255;
 green= (color.green>>8)&255;
 blue= (color.blue>>8)&255;
 rgb= (red<<16) | (green<<8) | blue;
 return(rgb);
}


int Xpppoem_change_color(w,colorname,flag)
Widget w;
char *colorname;
int flag;
{
  XrmValue namein, pixelout;
  Arg args[1];

  namein.addr= colorname;
  namein.size= strlen(colorname);
  XtConvert(w,XtRString, &namein, XtRPixel, &pixelout);
  if (pixelout.addr) {
/*
    fprintf(stderr,"\n new pixel value = %X\n",*(Pixel*)(pixelout.addr));
i*/
    XtSetArg(args[0], XtNbackground, *(Pixel*)(pixelout.addr));
    XtSetValues(w, args, 1);
  }
  return(1);
}


int Xpppoem_show_heat(w,heat,flag)
Widget  w;
double heat;   /* 0 to 1 */
int flag;
/*
 bit0 - bit3 = mode
           0 = compute and eventually show color
           1 = set colors[0] to 24bit RGB Value in heat
           2 = compute color scale from colors[0] and 24bit RGB Value in heat
 bit4= ignore last_idx
*/
{
 struct stat stbuf;
 int ret=1,idx,colorbase,colorscale,i,rgb,mode,debug= 0;
 int red_low,green_low,blue_low,red_high,green_high,blue_high;
 static char colors[][16]= {
                               "#a0a0a0", "#ae8e82", "#b2867e", "#b87e7a",
                               "#c07876", "#c87272", "#d06c6c", "#d86666",
                               "#e06060", "#e85a5a", "#f05454", "#ff4848",
                               "#ff3838", "#ff2424", "#ff1212", "#ff0000",
                           };
 static int last_idx= -1,count= 0;

 if(stat("xpppoem_debug_color_change",&stbuf)!=-1)
   debug= 1;
 if(debug)
   printf("Xpppoem_show_heat: count=%ld  flag=%ld  heat=%lf  last_idx=%ld\n",
          count,flag,heat,last_idx);
 count++;

 colorbase= 0; colorscale= 15;

 mode= flag&15;
 if(mode==1) {
   rgb= heat;
   sprintf(colors[0],"#%-6.6x",rgb);
   return(1);
 } else if(mode==2) {
   sscanf(colors[0]+1,"%X",&rgb);
   blue_low= rgb&255;
   green_low= (rgb>>8)&255;
   red_low= (rgb>>16)&255;
   rgb= heat;
   blue_high= rgb&255;
   green_high= (rgb>>8)&255;
   red_high= (rgb>>16)&255;
   for(i=1;i<colorscale;i++)
     sprintf(colors[i],"#%-2.2x%-2.2x%-2.2x",
                       red_low+(i*(red_high-red_low))/colorscale,
                       green_low+(i*(green_high-green_low))/colorscale,
                       blue_low+(i*(blue_high-blue_low))/colorscale);
   sprintf(colors[colorscale],"#%-2.2x%-2.2x%-2.2x",
                       red_high,green_high,blue_high);
   return(1);
 }

 if(heat<0.0)
   heat= 0.0;
 else if(heat>1.0)
   heat= 1.0;
 if(heat==0.0 || heat==1.0)
   idx= heat*colorscale;
 else
   idx= heat*(colorscale-1)+1;


 if(idx!=last_idx || (flag&16)) {
   ret= Xpppoem_change_color(w,colors[colorbase+idx],0);
   if(debug)
     printf("idx=%ld  changing colors  ret=%ld\n",idx,ret);
 } else {
   if(debug)
     printf("idx=%ld  no color change\n");
 } 
 last_idx= idx;
 return(ret);
}


int Xpppoem_report_open(w,mon,flag)
Widget	w;
struct PppoeM *mon;
int flag;
{
 struct stat stbuf;
 int ret;
 Arg args[1];

 if(mon->is_open>0) {
   /* text terminal stdout */
   ret= Pppoem_report(mon,"DSL ON",1|2);
   if(ret<=0)
     goto ex;
   if(! Xpppoem_do_show_heaT) {
     XtSetArg(args[0], XtNbackground, onlinecolor);
     XtSetValues(w, args, 1);
     if(stat("xpppoem_debug_color_change",&stbuf)!=-1)
       printf("Xpppoem_report_open: changing to ON color\n");
   }
 } else {
   /* text terminal stdout */
   ret= Pppoem_report(mon,"DSL OFF",0);
   if(ret<=0)
     goto ex;
   if(stat("xpppoem_debug_color_change",&stbuf)!=-1)
     printf("Xpppoem_report_open: changing to OFF color\n");
   XtSetArg(args[0], XtNbackground, offlinecolor );
   XtSetValues(w, args, 1);
 }
 ret= 1;
ex:;
 return(ret);
}


void Xpppoem_callback( w, closure, call_data )
Widget	w;
XtPointer closure;
XtPointer call_data;	/* pointer to (double) return value */
{
 int ret,was_open,is_open,hflag;
 struct PppoeM *mon;
 char line[160];
 double count,timespan,cps= 0.0;
 Arg args[1];

 mon= PppoemM;

 was_open= Pppoem_is_open(mon,0);
 ret= Pppoem_poll(mon,0);
 if(ret<=0)
   goto ex;
 is_open= Pppoem_is_open(mon,0);
 if(is_open!=was_open) {
   ret= Xpppoem_report_open(w,mon,0);
   if(ret<=0)
     goto ex;
 } else {
   /* text terminal stderr status line */
   ret= Pppoem_show_stat(mon,0);
   if(ret<=0)
     goto ex;
 }

   if(Xpppoem_show_labeL) {
     /* graphics window headline */
     ret= Pppoem_format_stat(mon,line,2|((is_open<=0)<<2)|8);
     if(ret<=0)
       goto ex;
     XtSetArg (args[0], XtNlabel, line);
     XtSetValues (label_wid, args, ONE);
   }
   /* graphics window histogram */
   ret= Pppoem_get_interval(mon,0,1,&count,&timespan,0);
   if(ret<0)
     goto ex;
   if(ret>0 && timespan>0)
     cps= count/timespan;
   if(Xpppoem_do_show_heaT && is_open) {
     /* Dynamic background color of graphics window histogram */
     ret= Pppoem_get_interval(mon,0,Xpppoem_busy_sampleS,&count,&timespan,0);
     if(ret<0)
       goto ex;
     hflag= (is_open!=was_open)<<4;
     if(ret>0 && timespan>0)
       Xpppoem_show_heat(w,count/timespan/Xpppoem_heat_rangE,hflag);
     else {
       ret= Pppoem_get_interval(mon,0,0,&count,&timespan,1);
       if(timespan>0)
         Xpppoem_show_heat(w,count/timespan/Xpppoem_heat_rangE,hflag);
       else
         Xpppoem_show_heat(w,0.0,hflag);
     }
   }

 if(is_open!=was_open && is_open>0 && Pppoem_has_headlines_on(mon,0))
   Pppoem_show_stat(mon,2);             /* new text terminal stderr headline */
 Pppoem_forward(mon,0);
 ret= 1;
ex:;
 if(ret<=0)
   exit(2);
 if(Xpppoem_show_grapH && is_open)
   *((double *)call_data) = cps / Xpppoem_uniT;    /* Xpppoem_uniT Bytes/sec */
 else
   *((double *)call_data) = 0.0;
 return;
}
        

int Xpppoem_main(argc, argv)
     int argc;
     char **argv;
{

#define Xpppoem_arg_leN 4096

  XtAppContext app_con;
  Widget toplevel, load, pane, load_parent;
  Pixmap icon_pixmap = None;
  XrmValue namein, pixelout;
  Arg args[1];
  char hargv_d[2][Xpppoem_arg_leN],*hargv[2];
  int ret,busy_color_enable= 0,i;

  hargv[0]= hargv_d[0];
  hargv[1]= hargv_d[1];

  ProgramName = argv[0];
    
  toplevel = XtAppInitialize(&app_con, "XPppoem", options,
			     XtNumber(options),
			     &argc, argv, NULL, NULL, (Cardinal) 0);

  XtGetApplicationResources( toplevel, (XtPointer) &resources,
			     my_resources, XtNumber(my_resources),
			     NULL, (Cardinal) 0);
  
  if (argc > 1) {
    for(i=1;i<argc;i++) {
      if(strcmp(argv[i],"-help")==0 || strcmp(argv[i],"-h")==0)
    break;
    }
    if(i<argc) {
      usage(0);
      exit(0);
    } else {
      usage(1);
      exit(1);
    }
  }

  Xpppoem_init(argv[1]);
    
  /*
   * This is a hack so that f.delete will do something useful in this
   * single-window application.
   */
  XtAppAddActions (app_con, xpppoem_actions, XtNumber(xpppoem_actions));
  XtOverrideTranslations(toplevel,
			 XtParseTranslationTable ("<Message>WM_PROTOCOLS: quit()"));

  XtSetArg (args[0], XtNiconPixmap, &icon_pixmap);
  XtGetValues(toplevel, args, ONE);
  if (icon_pixmap == None)
    {
      XtSetArg(args[0], XtNiconPixmap,
	       XCreateBitmapFromData(XtDisplay(toplevel),
				     XtScreen(toplevel)->root,
				     (char *)xpppload_bits,
				     xpppload_width, xpppload_height));
      XtSetValues (toplevel, args, ONE);
    }
    
  if (resources.show_label) {
      pane = XtCreateManagedWidget ("paned", panedWidgetClass,
				    toplevel, NULL, ZERO);
      
      XtSetArg (args[0], XtNlabel, "uninitialized");
      label_wid = XtCreateManagedWidget ("label", labelWidgetClass,
					 pane, args, 1);
      load_parent = pane;
    Xpppoem_show_labeL= 1;
  } else {
    Xpppoem_show_labeL= 0;
    load_parent = toplevel;
  }
  load = XtCreateManagedWidget ("load", stripChartWidgetClass,
				load_parent, NULL, ZERO);

  XtSetArg (args[0], XtNbackground, &bgcolor);
  XtGetValues(toplevel, args, ONE);
    
  if (resources.show_graph) {
    Xpppoem_show_grapH= 1;
  } else {  
    Xpppoem_show_grapH= 0;
  }

  if (resources.online_color)
    {
      char *colorname;
      unsigned rgb= 0xa0a0a0;

      colorname= resources.online_color;
      namein.addr = colorname;
      namein.size = strlen(colorname) + 1;
      XtConvert(load, XtRString, &namein, XtRPixel, &pixelout);
      if (!pixelout.addr) {
	fprintf(stderr, "could not convert online color `%s'.\n",colorname);
        exit(1);
      }
      onlinecolor = *(Pixel*)(pixelout.addr);
      if(0 && colorname[0]=='#') {
        sscanf(colorname+1,"%X",&rgb);
      } else {
        rgb= Xpppoem_pixel_to_rgb(load,onlinecolor,0);
        /* would work too :
          rgb= Xpppoem_colorname_to_rgb(load,colorname,0);
        */
        Xpppoem_show_heat(load,(double) rgb,1);
        busy_color_enable= 1;
      }
    }
  else 
    onlinecolor = bgcolor;
  
  if (resources.busy_color) {
    char *cpt;
    unsigned rgb= 0xff0000;

    cpt= resources.busy_color;
    if(strcmp(cpt,"off")==0) {
      Xpppoem_do_show_heaT= 0;
    } else {
      rgb= Xpppoem_colorname_to_rgb(load,resources.busy_color,0);
      Xpppoem_show_heat(load,(double) rgb,2);
      Xpppoem_do_show_heaT= 1;
    }
  }

  if (resources.offline_color)
    {
      namein.addr = resources.offline_color;
      namein.size = strlen(resources.offline_color) + 1;
      XtConvert(load, XtRString, &namein, XtRPixel, &pixelout);
      if (!pixelout.addr)
	{
	  fprintf(stderr, "could not convert offline color `%s'.\n",
		  resources.offline_color);
	  exit(1);
	}
      offlinecolor = *(Pixel*)(pixelout.addr);
    } 
  else 
    offlinecolor = bgcolor;

  if (resources.cps_range) {
    double value= 0.0;
    sscanf(resources.cps_range,"%lf",&value);
    if(value>1.0)
      Xpppoem_heat_rangE= value;
  }

  if (resources.show) {
    strcpy(hargv_d[0],"-show");
    strncpy(hargv_d[1],resources.show,Xpppoem_arg_leN-1);
    hargv_d[1][Xpppoem_arg_leN-1]= 0;
    ret= Pppoem_arg_interpreter(PppoemM,2,hargv,0);
    if(ret<=0)
      exit(1);
  }

  if (resources.busy_samples) {
    double value= 0.0;
    sscanf(resources.busy_samples,"%lf",&value);
    if(value>=1.0)
      Xpppoem_busy_sampleS= value;
  }

  if (resources.unit) {
    double value= 0.0;
    sscanf(resources.unit,"%lf",&value);
    if(Xpppoem_uniT>1.0)
      Xpppoem_uniT= value;
  }

  if (resources.dev_name) {
    strcpy(hargv_d[0],"-dev_name");
    strncpy(hargv_d[1],resources.dev_name,Xpppoem_arg_leN-1);
    hargv_d[1][Xpppoem_arg_leN-1]= 0;
    ret= Pppoem_arg_interpreter(PppoemM,2,hargv,0);
    if(ret<=0)
      exit(1);
  }

  if (resources.pretend) {
    strcpy(hargv_d[0],"-pretend");
    strncpy(hargv_d[1],resources.pretend,Xpppoem_arg_leN-1);
    hargv_d[1][Xpppoem_arg_leN-1]= 0;
    ret= Pppoem_arg_interpreter(PppoemM,2,hargv,0);
    if(ret<=0)
      exit(1);
  }

  if (resources.listen_stdin) {
    strcpy(hargv_d[0],"-listen_stdin");
    strncpy(hargv_d[1],resources.listen_stdin,Xpppoem_arg_leN-1);
    hargv_d[1][Xpppoem_arg_leN-1]= 0;
    ret= Pppoem_arg_interpreter(PppoemM,2,hargv,0);
    if(ret<=0)
      exit(1);
  }

  if (resources.report) {
    strcpy(hargv_d[0],"-report");
    strncpy(hargv_d[1],resources.report,Xpppoem_arg_leN-1);
    hargv_d[1][Xpppoem_arg_leN-1]= 0;
    ret= Pppoem_arg_interpreter(PppoemM,2,hargv,0);
    if(ret<=0)
      exit(1);
  }

  Xpppoem_start(PppoemM,0);

  XtAddCallback(load, XtNgetValue, Xpppoem_callback, NULL);

  XtRealizeWidget (toplevel);

  wm_delete_window = XInternAtom (XtDisplay(toplevel), "WM_DELETE_WINDOW",
				  False);
  (void) XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel),
			  &wm_delete_window, 1);

  XtAppMainLoop(app_con);

  return 0;
}



int main(argc, argv)
     int argc;
     char **argv;
{
  Xpppoem_main(argc,argv);
}


static void quit (w, event, params, num_params)
    Widget w;
    XEvent *event;
    String *params;
    Cardinal *num_params;
{
    if (event->type == ClientMessage &&
        event->xclient.data.l[0] != wm_delete_window) {
        XBell (XtDisplay(w), 0);
        return;
    }
    XtDestroyApplicationContext(XtWidgetToApplicationContext(w));
    exit (0);
}
